// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) ZenJu (zhnmju123 AT gmx DOT de) - All Rights Reserved    *
// **************************************************************************

#include <iostream>
#include <fstream>
#include <zen/com_error.h>
#include <zen/scope_guard.h>
#include "shadow.h"
#include "ObjBase.h"

using namespace shadow;


int main(int argc, wchar_t* argv[])
{
    std::wstring df = zen::generateErrorMsg(L"", 0x80042316L); //== VSS_E_SNAPSHOT_SET_IN_PROGRESS

    //create test file
    const wchar_t* testFileName = L"C:\\locked.txt";
    std::ofstream testFile(testFileName);
    testFile << "sample text" << std::endl;
    testFile.close();

    //obtain exclusive lock on test file
    HANDLE hFile = ::CreateFile(testFileName,
                                GENERIC_READ,
                                0, //no sharing!
                                nullptr,
                                OPEN_EXISTING,
                                FILE_FLAG_BACKUP_SEMANTICS,
                                nullptr);
    if (hFile == INVALID_HANDLE_VALUE)
    {
        std::cout << "error obtaining exclusive lock on test file: " << std::endl;
        return -1;
    }
    ZEN_ON_SCOPE_EXIT(::CloseHandle(hFile));

    //make shadow copy
    ::CoInitializeEx(nullptr, 0 );
    ZEN_ON_SCOPE_EXIT(CoUninitialize(););

    wchar_t message[1000];
    ShadowHandle backupHandle = createShadowCopy(L"C:\\", message, 1000);
    if (!backupHandle)
    {
        MessageBox(0, message, L"", 0);
        return -1;
    }
    ZEN_ON_SCOPE_EXIT(releaseShadowCopy(backupHandle));

    wchar_t shadowVolName[1000];
    getShadowVolume(backupHandle, shadowVolName, 1000);

    //now copying should be possible using the shadow copy service
    std::wstring shadowFile(shadowVolName);
    shadowFile += L"\\locked.txt";
    ::CopyFile(shadowFile.c_str(),
               L"C:\\copyLocked.txt",
               true);

    return 0;
}

