// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) ZenJu (zhnmju123 AT gmx DOT de) - All Rights Reserved    *
// **************************************************************************

#include <iostream>
#include <fstream>
#include <string>
#include <iterator>

int main(int args, char* argv[])
{
    if (args != 3)
    {
        std::cout << "Please specify a source filename as %1 parameter and target as %2!" << std::endl;
        return -1;
    }

    //read input file into buffer, if existing
    std::ifstream inputFile(argv[1], std::ios::binary);
    if (!inputFile)
    {
        std::cout << "Error opening " << argv[1] << "!" << std::endl;
        return -1;
    }
    //read input file completely
    inputFile >> std::noskipws; //[!]
    std::string inputStream((std::istream_iterator<char>(inputFile)), std::istream_iterator<char>());
    inputFile.close();

    //skip UTF-8 Byte Ordering Mark
    const bool hasBOM = inputStream.size() >= 3 && inputStream.substr(0, 3) == "\xEF\xBB\xBF";
    if (hasBOM)
        inputStream = inputStream.substr(3);

    //write file without BOM
    std::ofstream outputfile(argv[2], std::ios::binary);
    if (!outputfile)
    {
        std::cout << "Error writing " << argv[2] << "!" << std::endl;
        return -1;
    }

    outputfile.write(inputStream.c_str(), inputStream.size());

    std::cout << "\nOperation successful!" << std::endl;
    return 0;
}
